@@ -3,6 +3,13 @@ import pkg_resources
 from repoze.configuration.loader import PluginLoader
 
 import os
+import re
+import sys
+
+_KEYCRE = re.compile(r"%\(([^)]*)\)s")
+
+class ConfigurationError(Exception):
+    pass
 
 class Context(object):
     def __init__(self, registry):
@@ -11,16 +18,32 @@ class Context(object):
         self.stack = []
         self.discriminators = {}
 
+    def interpolate(self, value):
+        def _interpolation_replace(match):
+            s = match.group(1)
+            return self.registry[s]
+        if '%(' in value:
+            value = _KEYCRE.sub(_interpolation_replace, value)
+        return value
+
     def action(self, info, node):
         discriminator = info['discriminator']
         callback = info['callback']
-        if self.stack and not self.stack[-1]['override']:
+        override = info.get('override')
+        stack_override = self.stack and self.stack[-1]['override']
+        effective_override = override or stack_override
+        if not effective_override:
             if discriminator in self.discriminators:
                 conflicting_action = self.discriminators[discriminator]
                 raise ConfigurationConflict(node, conflicting_action.node)
-                
-        self.discriminators
-        self.actions.append(Action(discriminator, callback, node))
+
+        action = Action(discriminator, callback, node)
+        self.actions.append(action)
+        self.discriminators[discriminator] = action
+
+    def error(self, node, msg):
+        msg = '%s\n%s' % (msg , lineinfo(node))
+        raise ConfigurationError(msg)
 
     def resolve(self, dottedname):
         if dottedname.startswith('.') or dottedname.startswith(':'):
@@ -28,7 +51,10 @@ class Context(object):
             if not package:
                 raise ValueError('name "%s" is irresolveable (no package)' %
                                  dottedname)
-            dottedname = package.__name__ + dottedname
+            if dottedname in ['.', ':']:
+                dottedname = package.__name__
+            else:
+                dottedname = package.__name__ + dottedname
         try:
             return pkg_resources.EntryPoint.parse(
                 'x=%s' % dottedname).load(False)
@@ -87,6 +113,15 @@ class Context(object):
                 name, type.__name__, value))
         return value
 
+    def popvalue(self, structure, name, default=None, type=basestring):
+        value = structure.pop(name, default)
+        if value is default:
+            return default
+        if not isinstance(value, type):
+            raise ValueError('"%s" attribute is not a %s: %r' % (
+                name, type.__name__, value))
+        return value
+
     def call_later(self, func, *arg, **kw):
         def callback():
             return func(*arg, **kw)
@@ -99,7 +134,13 @@ class Action(object):
         self.node = node
         
     def execute(self):
-        self.callback()
+        try:
+            self.callback()
+        except Exception, why:
+            exc_info = sys.exc_info()
+            msg = 'While executing %s' % lineinfo(self.node)
+            why.args += (msg,)
+            raise exc_info[0], why, exc_info[2]
 
 class ConfigurationConflict(Exception):
     def __init__(self, node1, node2):
@@ -109,20 +150,29 @@ class ConfigurationConflict(Exception):
 
     def __str__(self):
         message = []
-        message.append('Conflicting declarations: ')
-        message.append(self._lineinfo(self.node1))
+        message.append('Conflicting declarations:')
+        message.append(lineinfo(self.node1))
         message.append('conflicts with')
-        message.append(self._lineinfo(self.node2))
-        return '\n'.join(message)
+        message.append(lineinfo(self.node2))
+        return '\n\n'.join(message)
 
-    def _lineinfo(self, node):
-        msg = ('lines %s:%s-%s:%s of file "%s"' % (
-            node.start_mark.line,
-            node.start_mark.column,
-            node.end_mark.line,
-            node.end_mark.column,
-            node.start_mark.name,
-            )
+def lineinfo(node):
+    start_mark = node.start_mark
+    end_mark = node.end_mark
+    filename = start_mark.name
+    try:
+        f = open(filename, 'r')
+        f.seek(start_mark.index)
+        data = f.read(end_mark.index - start_mark.index) + 'in '
+    except (OSError, IOError):
+        data = ''
+    msg = ('%slines %s:%s-%s:%s of file "%s"' % (
+        data,
+        start_mark.line,
+        start_mark.column,
+        end_mark.line,
+        end_mark.column,
+        start_mark.name,
         )
-        return msg
-
+    )
+    return msg
